# coding=utf-8
from unittest import TestCase
from zpretty.text import lstrip_first_line
from zpretty.text import rstrip_last_line


class TestFunctions(TestCase):
    ''' Test functions used by zpretty
    '''

    def test_lstrip_first_line_oneline(self):
        self.assertEqual(
            lstrip_first_line(u' a'), u'a'
        )

    def test_lstrip_first_line_twolines(self):
        self.assertEqual(
            lstrip_first_line(u' a \n b'),
            (u'a \n b')
        )

    def test_rstrip_larst_line_oneline(self):
        self.assertEqual(
            rstrip_last_line(u'a'), u'a'
        )

    def test_rstrip_larst_line_twoline(self):
        self.assertEqual(
            rstrip_last_line(u'a\n b '),
            (u'a\n b')
        )

    def test_none(self):
        self.assertEqual(lstrip_first_line(None), None)
        self.assertEqual(rstrip_last_line(None), None)



# coding=utf-8
from bs4 import BeautifulSoup
from unittest import TestCase
from zpretty.elements import OpenTagException
from zpretty.elements import PrettyElement


class TestPrettyElements(TestCase):
    ''' Test basic funtionalities of the PrettyElement class
    '''
    def get_element(self, text):
        ''' Given a text return a PrettyElement
        '''
        soup = BeautifulSoup(
            u'<soup><fake_root>%s</fake_root></soup>' % text,
            'html.parser'
        )
        return PrettyElement(soup.fake_root.next_element)

    def test_comment(self):
        el = self.get_element('<!--a-->')
        self.assertFalse(el.is_processing_instruction())
        self.assertFalse(el.is_doctype())
        self.assertFalse(el.is_tag())
        self.assertFalse(el.is_text())
        self.assertTrue(el.is_comment())
        self.assertEqual(el.text, u'a')
        self.assertEqual(el.getchildren(), [])
        self.assertEqual(el.attributes(), u'')
        self.assertEqual(el(), u'<!--a-->')

    def test_text_element(self):
        el = self.get_element('text<ignored_children>')
        self.assertFalse(el.is_comment())
        self.assertFalse(el.is_doctype())
        self.assertFalse(el.is_processing_instruction())
        self.assertFalse(el.is_tag())
        self.assertTrue(el.is_text())
        self.assertEqual(el.text, u'text')
        self.assertEqual(el.getchildren(), [])
        self.assertEqual(el.attributes(), u'')
        self.assertEqual(el(), u'text')

    def test_empty_element(self):
        el = self.get_element('<root />')
        self.assertFalse(el.is_comment())
        self.assertFalse(el.is_doctype())
        self.assertFalse(el.is_processing_instruction())
        self.assertFalse(el.is_text())
        self.assertTrue(el.is_tag())
        self.assertEqual(el.text, u'')
        self.assertEqual(el.getchildren(), [])
        self.assertEqual(el.attributes(), u'')

    def test_empty_element_attributes(self):
        el = self.get_element('<root class="b" />')
        self.assertFalse(el.is_comment())
        self.assertFalse(el.is_doctype())
        self.assertFalse(el.is_processing_instruction())
        self.assertFalse(el.is_text())
        self.assertTrue(el.is_tag())
        self.assertEqual(el.text, u'')
        self.assertEqual(el.getchildren(), [])
        self.assertEqual(el.attributes(), u'class="b"')
        self.assertEqual(el(), u'<root class="b"></root>')

    def test_processing_instruction(self):
        el = self.get_element(u'<?xml version="1.0" encoding="utf-8">')
        self.assertFalse(el.is_tag())
        self.assertFalse(el.is_text())
        self.assertFalse(el.is_doctype())
        self.assertTrue(el.is_processing_instruction())
        self.assertEqual(el.text, u'xml version="1.0" encoding="utf-8"')
        self.assertEqual(el.getchildren(), [])
        self.assertEqual(el.attributes(), u'')

    def test_doctype(self):
        el = self.get_element(u'<!DOCTYPE html>')
        self.assertFalse(el.is_tag())
        self.assertFalse(el.is_text())
        self.assertFalse(el.is_processing_instruction())
        self.assertTrue(el.is_doctype())
        self.assertEqual(el.text, u'html')
        self.assertEqual(el.getchildren(), [])
        self.assertEqual(el.attributes(), u'')
        el = self.get_element(
            u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" '
            u'"http://www.w3.org/TR/html4/strict.dtd">'
        )
        self.assertEqual(
            el.text,
            u'HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" '
            u'"http://www.w3.org/TR/html4/strict.dtd"'
        )

    def test_render_text(self):
        el = self.get_element(' a')
        self.assertEqual(el.render_text(), u'\na')
        el = self.get_element(' ')
        self.assertEqual(el.render_text(), u'\n')
        el = self.get_element('\n')
        self.assertEqual(el.render_text(), u'\n')

    def test_is_self_closing_open_raises(self):
        ''' If we have a known self closing element not closed,
        we raise an exception
        '''
        self.assertRaises(
            OpenTagException,
            self.get_element('<root><input><div /></root>'),
        )



# coding=utf-8
from cgi import escape
from logging import getLogger


logger = getLogger(__name__)


class PrettyAttributes(object):
    ''' Render attributes in a pretty way.

    - one per line
    - sorted semantically and alphabetically
    - with properly indented and escaped values
    '''

    _attribute_template = u'%s="%s"'
    _valueless_attributes_are_allowed = True
    _known_valueless_attributes = (
        'hidden',
        'required',
    )
    _multiline_prefix = u'  '
    _multiline_attributes = (
        u'attributes',
        u'define',
        u'tal:attributes',
        u'tal:define',
    )

    _tal_attribute_order = (
        'tal:define',
        'tal:switch',
        'tal:condition',
        'tal:repeat',
        'tal:case',
        'tal:content',
        'tal:replace',
        'tal:omit-tag',
        'tal:attributes',
        'tal:on-error',
    )

    _i18n_attributes = (
        'i18n:translate',
        'i18n:domain',
        'i18n:context',
        'i18n:source',
        'i18n:target',
        'i18n:name',
        'i18n:attributes',
        'i18n:data',
        'i18n:comment',
        'i18n:ignore',
        'i18n:ignore-attributes',
    )

    def __init__(self, attributes):
        ''' attributes is a dict like object
        '''
        self.attributes = attributes

    def sort_attributes(self, name):
        '''This sorts the attribute trying to group theme semantically

        Starting from the top:

        1. xml namespaces
        2. class, id
        3. attributes not belonging in to other categories (default)
        4. data- attributes
        5. tal attributes
        6. i18n attributes
        '''
        if name.startswith('xmlns'):
            return (0, name)
        if name in ('class', 'id'):
            return (100, name)
        if name.startswith('data'):
            return (300, name)
        if 'tal:' + name in self._tal_attribute_order:
            tal_index = self._tal_attribute_order.index('tal:' + name)
            return (400 + tal_index, name)
        if name in self._tal_attribute_order:
            tal_index = self._tal_attribute_order.index(name)
            return (400 + tal_index, name)
        if name in self._i18n_attributes:
            return (900, name)
        return (200, name)

    def format_tal_multiline(self, value):
        ''' There are some tal specific attributes that contain ; separated
        statements.
        They are used to define variables or set other attributes.
        You can define many variables by adding statements separated by ';'.
        If the statement contains a ';', it will be escaped as ';;'.

        It is convenient to always have those attribute in the form:

        tal:define="
          var1 statement1;
          var2 statement2;
          ...
        "
        '''
        # temp skip ';;' the escape sequence to enter a ';' in a statement
        statements = value.replace(';;', '<>').split(';')

        # We always want an empty line first...
        lines = [u'']
        for statement in statements:
            statement = statement.strip()
            if statement:
                if not statement.endswith(';'):
                    statement += u';'
                lines.append(
                    self._multiline_prefix + statement
                )
        # ... and at the end
        lines.append(u'')

        new_value = u'\n'.join(lines)
        # restore ';;'
        return new_value.replace('<>', ';;')

    def can_be_valueless(self, name):
        ''' Check if the attribute name can be without a value
        '''
        if not self._valueless_attributes_are_allowed:
            return False
        if name.startswith('data-'):
            return True
        if name in self._known_valueless_attributes:
            return True
        return False

    def lines(self):
        ''' Take the attributes, sort them and prettify their values
        '''
        attributes = self.attributes
        sorted_names = sorted(attributes, key=self.sort_attributes)
        lines = []
        for name in sorted_names:
            value = attributes[name]
            if isinstance(value, list):
                # Happens, e.g., for the class attribute
                value = u' '.join(value)
            if name in self._multiline_attributes:
                value = self.format_tal_multiline(value)
            if not value and self.can_be_valueless(name):
                line = name
            else:
                line = self._attribute_template % (name, escape(value))
            lines.append(line)
        return lines

    def __call__(self):
        ''' Render the attributes as text
        '''
        return u'\n'.join(self.lines())



# coding=utf-8
from pkg_resources import resource_filename
from unittest import TestCase
from zpretty.prettifier import ZPrettifier


class TestZpretty(TestCase):
    ''' Test zpretty
    '''
    maxDiff = None

    def assertPrettified(self, original, expected):
        ''' Check if the original html has been prettified as expected
        '''
        if isinstance(expected, tuple):
            expected = u'\n'.join(expected)
        prettifier = ZPrettifier(text=original)
        observed = prettifier()
        self.assertEqual(observed, expected)

    def prettify(self, filename):
        ''' Run prettify on filename and check that the output is equal to
        the file content itself
        '''
        resolved_filename = resource_filename(
            'zpretty.tests',
            'original/%s' % filename
        )
        prettifier = ZPrettifier(resolved_filename)
        observed = prettifier()
        expected = open(resolved_filename).read().decode('utf8')
        self.assertListEqual(
            observed.splitlines(),
            expected.splitlines(),
        )

    def test_format_self_closing_tag(self):
        self.assertPrettified('<tal:test />', u'<tal:test />')

    def test_nesting_no_text(self):
        # no attributes
        self.assertPrettified(
            '<root><tal:test /></root>',
            u'<root><tal:test /></root>',
        )

        self.assertPrettified(
            '<root><tal:test /> </root>',
            u'<root><tal:test />\n</root>',
        )

        self.assertPrettified(
            '<root> <tal:test /> </root>',
            u'<root>\n  <tal:test />\n</root>',
        )

        self.assertPrettified(
            '<root> <tal:test /></root>',
            u'<root>\n  <tal:test /></root>',
        )
        self.assertPrettified(
            '<root><tal:test><child /></tal:test></root>',
            u'<root><tal:test><child></child></tal:test></root>',
        )

    def test_nesting_with_text(self):
        # no attributes
        self.assertPrettified(
            '<root> a<tal:test /></root>',
            u'<root>\n  a<tal:test /></root>',
        )

        self.assertPrettified(
            '<root>a <tal:test /> </root>',
            u'<root>a\n  <tal:test />\n</root>',
        )

        self.assertPrettified(
            '<root> a <tal:test /> </root>',
            u'<root>\n  a\n  <tal:test />\n</root>',
        )

        self.assertPrettified(
            '<root> a <div /> </root>',
            u'<root>\n  a\n  <div></div>\n</root>',
        )
        self.assertPrettified(
            '<root> a <div /> <div /> </root>',
            u'<root>\n  a\n  <div></div>\n  <div></div>\n</root>',
        )
        self.assertPrettified(
            '<root> a <div />\n <div /> </root>',
            u'<root>\n  a\n  <div></div>\n  <div></div>\n</root>',
        )
        self.assertPrettified(
            '<root> a <div />\na <div /> </root>',
            u'<root>\n  a\n  <div></div>\na\n  <div></div>\n</root>',
        )
        self.assertPrettified(
            '<root> <div>a</div> </root>',
            u'<root>\n  <div>a</div>\n</root>',
        )
        self.assertPrettified(
            '<root> <div>a\n  </div> </root>',
            u'<root>\n  <div>a\n  </div>\n</root>',
        )

    def test_nesting_with_tail(self):
        # no attributes
        self.assertPrettified(
            '<root><p></p><!-- #a -->\n\n<p></p></root>',
            u'<root><p></p><!-- #a -->\n  <p></p></root>',
        )
        self.assertPrettified(
            '<root><tal:test />a</root>',
            u'<root><tal:test />a</root>',
        )
        self.assertPrettified(
            '<root><tal:test />a </root>',
            (
                u'<root><tal:test />a',
                u'</root>',
            )
        )

        self.assertPrettified(
            '<root><tal:test /> a </root>',
            (
                u'<root><tal:test />',
                u'  a',
                u'</root>',
            )
        )
        self.assertPrettified(
            '<root><tal:test /> a </root>',
            u'<root><tal:test />\n  a\n</root>',
        )

    def test_many_children(self):
        '''
        '''
        self.assertPrettified(
            u'<root><tal:test /><div></div></root>',
            u'<root><tal:test /><div></div></root>',
        )
        self.assertPrettified(
            u'<root><tal:test /> <div></div></root>',
            u'<root><tal:test />\n  <div></div></root>',
        )

    def test_valueless_attributes(self):
        ''' Test attributes wthout value
        (hidden, required, data-attributes, ...)
        Some of them are rendered valueless, some other not.
        '''
        self.assertPrettified(
            u'<root data-attribute=""></root>',
            u'<root data-attribute></root>',
        )
        self.assertPrettified(
            u'<root hidden></root>',
            u'<root hidden></root>',
        )
        self.assertPrettified(
            u'<root selected></root>',
            u'<root selected=""></root>',
        )

    def test_element_repr(self):
        prettifier = ZPrettifier(text='')
        self.assertEqual(
            repr(prettifier.root),
            '<pretty:-1:null_tag_name />',
        )

    def test_whitelines_not_stripped(self):
        self.assertPrettified(
            u'<root>\n</root>',
            u'<root>\n</root>',
        )
        self.assertPrettified(
            u'<root>\n    Hello!   \n</root>',
            u'<root>\n    Hello!\n</root>',
        )

    def test_sample_xml(self):
        self.prettify('sample_xml.xml')

    def test_sample_html(self):
        self.prettify('sample_html.html')

    def test_text_with_markup(self):
        self.prettify('text_with_markup.md')



# coding=utf-8
from unittest import TestCase
from zpretty.attributes import PrettyAttributes


class TestZPrettyAttributess(TestCase):
    ''' Test zpretty
    '''
    def assertPrettifiedAttributes(self, attributes, expected, level=0):
        ''' Check if the attributes are properly sorted and formatted
        '''
        pretty_attribute = PrettyAttributes(attributes)
        observed = pretty_attribute()
        self.assertEqual(observed, expected)

    def test_no_attributes(self):
        self.assertPrettifiedAttributes(
            {},
            u''
        )

    def test_one_attribute(self):
        self.assertPrettifiedAttributes(
            {
                u'a': u'1',
            },
            u'a="1"',
        )

    def test_tal_define(self):
        self.assertPrettifiedAttributes(
            {
                u'tal:define': u'a 1; b 2',
            },
            u'\n'.join((
                u'tal:define="',
                u'  a 1;',
                u'  b 2;',
                u'"',
            )),
        )

    def test_format_attributes_many_attribute(self):
        self.assertPrettifiedAttributes(
            {
                u'a': u'1',
                u'b': u'2',
                u'class': u'hidden',
                u'tal:define': u'a 1; b 2',
            },
            u'\n'.join((
                u'class="hidden"',
                u'a="1"',
                u'b="2"',
                u'tal:define="',
                u'  a 1;',
                u'  b 2;',
                u'"',
            )),
        )



# coding=utf-8
from bs4 import BeautifulSoup
from logging import getLogger
from zpretty.elements import PrettyElement


logger = getLogger(__name__)


class ZPrettifier(object):
    ''' Wraps and renders some text that may contain xml like stuff
    '''

    encoding = 'utf8'
    pretty_element = PrettyElement

    def __init__(
        self,
        filename='',
        text='',
    ):
        ''' Create a prettifier instance taking the contents
        from a text or a filename
        '''
        self.filename = filename
        if self.filename:
            text = open(self.filename).read()
        if not isinstance(text, unicode):
            text = text.decode(self.encoding)
        self.text = text
        self.soup = self.get_soup(self.text)
        self.root = self.pretty_element(self.soup, -1)

    def get_soup(self, text):
        ''' Tries to get the soup from the given test

        At first it will try to parse the text:

        1. as an xml
        2. as an html
        3. will just return the unparsed text
        '''
        markup = u'<{null}>{text}</{null}>'.format(
            null=self.pretty_element.null_tag_name,
            text=text,
        )
        wrapped_soup = BeautifulSoup(markup, 'html.parser')
        return getattr(wrapped_soup, self.pretty_element.null_tag_name)

    def autofix(self):
        ''' Do various autofx on the soup
        '''

    def pretty_print(self, el):
        ''' Pretty print an element indenting it based on level
        '''
        return el()

    def __call__(self):
        self.autofix()
        return self.pretty_print(self.root)



# coding=utf-8


def startswith_whitespace(text):
    ''' Check if text starts with a whitespace

    If text is not a string return False
    '''
    if not isinstance(text, basestring):
        return False
    return text[:1].isspace()


def endswith_whitespace(text):
    ''' Check if text ends with a whitespace

    If text is not a string return False
    '''
    if not isinstance(text, basestring):
        return False
    return text[-1:].isspace()


def lstrip_first_line(text):
    ''' lstrip only the first line of text
    '''
    if not text:
        return text
    if endswith_whitespace(text):
        text += u' '
    lines = text.splitlines()
    lines[0] = lines[0].lstrip()
    return u'\n'.join(lines)


def rstrip_last_line(text):
    ''' rstrip only the last line of text
    '''
    if not text:
        return text
    if endswith_whitespace(text):
        text += u' '
    lines = text.splitlines()
    lines[-1] = lines[-1].rstrip()
    return u'\n'.join(lines)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

import os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup(
    name='zpretty',
    version='0.9.0',
    description='An opinionated HTML/XML soup formatter',
    keywords=[
        'Formatter',
        'HTML',
        'Prettifier',
        'Pretty print',
        'TAL',
        'XML',
        'ZPT',
    ],
    long_description='\n'.join((
        read('README.rst'),
        read('HISTORY.rst'),
    )),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only',
    ],
    author='Alessandro Pisa',
    author_email='alessandro.pisa@gmail.com',
    url='https://github.com/collective/zpretty',
    license='BSD',
    packages=find_packages(),
    install_requires=[
        'setuptools',
        'beautifulsoup4',
    ],
    extras_require={
        'test': [
            'nose',
            'nose-selecttests',
            'rednose',
            'coverage',
            'unittest2',
            'flake8',
        ],
        'development': [
            'zest.releaser',
            'check-manifest',
            'pyroma',
        ],
    },
    test_suite="zpretty.tests",
    entry_points={
        'console_scripts': [
            'zpretty=zpretty.cli:run',
        ]
    },
    include_package_data=True,
    zip_safe=False,
)



# coding=utf-8
from argparse import ArgumentParser
from sys import stdout
from zpretty.prettifier import ZPrettifier


def get_parser():
    parser = ArgumentParser(
        prog='zpretty',
        description='An opinionated HTML/XML soup formatter',
        epilog=None,
    )
    parser.add_argument(
        '-i',
        '--inplace',
        help='Format in place (overwrite existing file)',
        action='store_true',
        dest='inplace',
    )
    parser.add_argument(
        'file',
        nargs='*',
        default=None,
        help='The list of files to prettify',
    )
    return parser


def run():
    ''' Prettify each filename passed in the command line
    '''
    parser = get_parser()
    config = parser.parse_args()
    for infile in config.file:
        prettifier = ZPrettifier(infile)
        prettified = prettifier().encode('utf8')
        if config.inplace:
            with open(infile, 'w') as f:
                f.write(prettified)
        else:
            stdout.write(prettified)


if __name__ == '__main__':
    run()
